#ifndef	_DPORT_MM_
#define	_DPORT_MM_

#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdlib.h>

/**
 * 网卡配置
 */
typedef	struct _net_cfg{
	uint8_t ip[4];
	uint8_t mask[4];
	uint8_t gateway[4];
}net_cfg_t;

/**
 * 网络模式
 */
typedef	enum _net_mode{
	MODE_COMMON,  /**< MODE_COMMON */
	MODE_BONDING, /**< MODE_BONDING */
	MODE_BRIDGEIF,/**< MODE_BRIDGEIF */
}net_mode_t;

/**
 * net_mode_t的映射
 */
static char *mode_str[] = {
	"common",
	"bonding",
	"bridgeif",
};

/**
 * 串口数据位长度
 */
typedef	enum _uart_bytesize{
	UART_BYTESIZE_5 = 5,/**< UART_BYTESIZE_5 */
	UART_BYTESIZE_6,/**< UART_BYTESIZE_6 */
	UART_BYTESIZE_7,/**< UART_BYTESIZE_7 */
	UART_BYTESIZE_8,/**< UART_BYTESIZE_8 */
}uart_bytesize_t;

/**
 * uart_bytesize_t的映射
 */
static int uart_bytesize_map[] = {
	0,
	0,
	0,
	0,
	0,
	5,
	6,
	7,
	8,
};

/**
 * 串口校验位
 */
typedef	enum _uart_parity{
	UART_PARITY_NONE = 0,/**< UART_PARITY_NONE */
	UART_PARITY_ODD, /**< UART_PARITY_ODD */
	UART_PARITY_EVEN,/**< UART_PARITY_EVEN */
}uart_parity_t;

/**
 * uart_parity_t的映射
 */
static int uart_parity_map[] = {
	0,
	1,
	2,
};

/**
 * 串口停止位
 */
typedef	enum _uart_stopbits{
	UART_STOPBITS_1 = 1,  /**< UART_STOPBITS_1 */
	UART_STOPBITS_1_5,/**< UART_STOPBITS_1_5 */
	UART_STOPBITS_2,  /**< UART_STOPBITS_2 */
}uart_stopbits_t;

/**
 * uart_stopbits_t映射
 */
static int uart_stopbits_map[] = {
	0,
	1,
	3,
	2,
};

/**
 * 串口配置
 */
typedef	struct _uart_cfg{
	int baudrate;
	uart_bytesize_t bytesize;
	uart_parity_t parity;
	uart_stopbits_t stopbits;
}uart_cfg_t;

/**
 * SOCKET模式
 */
typedef	enum _socket_mode{
	SOCKET_UDP,          /**< SOCKET_UDP */
	SOCKET_UDP_MULTICAST,/**< SOCKET_UDP_MULTICAST */
	SOCKET_TCP_CLIENT,   /**< SOCKET_TCP_CLIENT */
	SOCKET_TCP_SERVER,   /**< SOCKET_TCP_SERVER */
}socket_mode_t;

static int socket_mode_map[] = {
	0,
	1,
	2,
	3,
};

/**
 * 使能/失能
 */
typedef	enum _enable{
	ENABLE_TRUE, /**< ENABLE_TRUE */
	ENABLE_FALSE,/**< ENABLE_FALSE */
}enable_t;

static char* enable_str_map[] = {
	"true",
	"false",
};


typedef	struct _socket_cfg{
	socket_mode_t mode;
	uint8_t host[4];
	uint8_t multicast[4];
	enable_t	enable;
	int port;
	int max_client_nr;
	int keepalive_time;
	int idle_down_time;
	int retry_interval;
}socket_cfg_t;

typedef	enum _transparency_mode{
	MODE_TRANSPARENCY,
	MODE_FRAME,
}transparency_mode_t;

static char* transparency_str_map[] = {
	"transparency",
	"frame",
};

typedef	enum _transparency_atcmd{
	ATCMD_ENABLE,
	ATCMD_DISABLE,
}transparency_atcmd_t;

static char* atcmd_str_map[] = {
	"Enable",
	"Disable",
};


#define	PORT_UART1  (1 << 0)
#define	PORT_UART2  (1 << 1)
#define	PORT_SOCKET1  (1 << 2)
#define	PORT_SOCKET2  (1 << 3)
#define	PORT_SOCKET3  (1 << 4)
#define	PORT_SOCKET4  (1 << 5)
#define	PORT_SOCKET5  (1 << 6)
#define	PORT_SOCKET6  (1 << 7)
#define	PORT_SOCKET7  (1 << 8)
#define	PORT_SOCKET8  (1 << 9)
#define	PORT_SOCKET9  (1 << 10)
#define	PORT_SOCKET10  (1 << 11)
#define	PORT_SOCKET11  (1 << 12)
#define	PORT_SOCKET12  (1 << 13)
#define	PORT_SOCKET13  (1 << 14)
#define	PORT_SOCKET14  (1 << 15)

typedef int transparency_port_t;
/**
 * transparency_tar_t：发送目标
 * 		transparency_port_t类型的或。
 * 		PORT_UART1|PORT_SOCKET1|PORT_SOCKET2|PORT_SOCKET3
 */
typedef int transparency_tar_t;

typedef	struct _transparency_cfg{
	transparency_mode_t mode;
	transparency_atcmd_t at_cmd;
	transparency_tar_t tar_number;
	int pack_size;
	int time_out;
}transparency_cfg_t;


/**
 * 设备创建
 * @param name	名称
 * @return	成功返回句柄，失败返回负数
 */
int dev_creat(char *name);
	
/**
 * 设备读数据
 * @param fd	句柄
 * @param buf	缓存
 * @param len	长度
 * @return	成功返回读到的长度，失败返回负数
 */
int dev_read(int fd, uint8_t *buf, uint32_t len);


/**
 * 设备写数据
 * @param fd	句柄
 * @param buf	缓存
 * @param len	长度
 * @return 成功返回写入的长度，失败返回负数
 */
int dev_write(int fd, uint8_t *buf, uint32_t len);


/**
 * 设备帧结构读数据
 * @param fd	句柄
 * @param port 源端口
 * @param buf	缓存
 * @param len	长度
 * @return	成功返回读到的长度，失败返回负数
 */
int dev_frame_read(int fd, transparency_port_t *port, uint8_t *buf, uint32_t len);

/**
 * 设备帧结构写数据
 * @param fd	句柄
 * @param tar	目标端口
 * @param buf	缓存
 * @param len	长度
 * @return 成功返回写入的长度，失败返回负数
 */
int dev_frame_write(int fd, transparency_tar_t tar, uint8_t *buf, uint32_t len);
	
/**
 * 进入协议模式
 * @param fd
 * @return	成功返回0,失败返回负数
 */
int dp_enter(int fd);

/**
 * 退出协议模式
 * @param fd
 * @return 成功返回0,失败返回负数
 */
int dp_exit(int fd);

/**
 * 设置模式
 * @param fd	句柄
 * @param mode	模式
 * @return 成功返回0，失败返回负数
 */
int dp_set_mode(int fd, net_mode_t mode);

/**
 * 获取模式
 * @param fd	句柄
 * @param mode	模式
 * @return	成功返回0，失败返回负数
 */
int dp_get_mode(int fd, net_mode_t *mode);

/**
 *	设置网卡信息
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_set_net(int fd, int index, net_cfg_t *cfg);

/**
 * 获取网卡信息
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_get_net(int fd, int index, net_cfg_t *cfg);

/**
 * 设置串口
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return
 */
int dp_set_uart(int fd, int index, uart_cfg_t *cfg);

/**
 * 获取串口配置
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return
 */
int dp_get_uart(int fd, int index, uart_cfg_t *cfg);

/**
 * 设置socket
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_set_socket(int fd, int index, socket_cfg_t *cfg);

/**
 * 获取socket配置
 * @param fd	句柄
 * @param index	序号
 * @param cfg	配置
 * @return	成功返回0，失败返回负数
 */
int dp_get_socket(int fd, int index, socket_cfg_t *cfg);

/**
 * 设置传输
 * @param fd	句柄
 * @param port	端口
 * @param cfg	配置
 * @return 成功返回0,失败返回负数
 */
int dp_set_transparency(int fd, transparency_port_t port, transparency_cfg_t *cfg);

/**
 * 获取传输端口配置
 * @param fd	句柄
 * @param port	端口
 * @param cfg	配置
 * @return	成功返回0,失败返回负数
 */
int dp_get_transparency(int fd, transparency_port_t port, transparency_cfg_t *cfg);


#endif
